@@ -10,6 +10,7 @@ Python :py:class:`argparse.Argumentparser` as well as custom :py:class:`argparse
 """
 
 import argparse
+import json
 
 from depthcharge import log
 from depthcharge.context    import Depthcharge
@@ -18,6 +19,9 @@ from depthcharge.console    import Console
 from depthcharge.monitor    import Monitor
 from depthcharge.string     import keyval_list_to_dict, length_to_int, str_to_property_keyval
 
+_DEFAULT_IFACE_DEVICE = '/dev/ttyUSB0'
+_DEFAULT_IFACE_BAUDRATE = '115200'
+_DEFAULT_IFACE = _DEFAULT_IFACE_DEVICE + ':' + _DEFAULT_IFACE_BAUDRATE
 
 def create_depthcharge_ctx(args, **kwargs):
     """
@@ -34,13 +38,14 @@ def create_depthcharge_ctx(args, **kwargs):
     * *iface* - From :py:meth:`ArgumentParser.add_interface_argument()`
     * *prompt* - From :py:meth:`ArgumentParser.add_prompt_argument()`
     * *extra* - From :py:meth:`ArgumentParser.add_extra_argument()`
-    * *skip_deploy* - From :py:meth:`ArgumentParser.add_skip_deploy_argument()`
 
     The following items are optional:
 
     * *config* - From :py:meth:`ArgumentParser.add_config_argument()`
     * *companion* - From :py:meth:`ArgumentParser.add_companion_argument()`
-
+    * *allow_deploy* - From :py:meth:`ArgumentParser.add_allow_deploy_argument()`
+    * *skip_deploy* - From :py:meth:`ArgumentParser.add_skip_deploy_argument()`
+    * *allow_reboot* - From :py:meth:`ArgumentParser.add_allow_reboot_argument()`
 
     """
     monitor = Monitor.create(args.monitor)
@@ -51,6 +56,19 @@ def create_depthcharge_ctx(args, **kwargs):
     # TODO: Mull this over a bit more, clean it up if needed, and document this.
     console_kwargs = kwargs.get('console_kwargs', {})
 
+    # Attempt to retrive baudrate from config. This is intended to make life
+    # less annoying when working with a device that uses the non-default
+    # baud rate and forgetting to provide -i <iface>, thereby relying on the
+    # /dev/ttyUSB0 default.
+    if args.iface == _DEFAULT_IFACE and hasattr(args, 'config') and args.config:
+        try:
+            with open(args.config) as infile:
+                config = json.loads(infile.read())
+                args.iface = _DEFAULT_IFACE_DEVICE + ':' + str(config['baudrate'])
+        except (KeyError, FileNotFoundError):
+            # No worries! Keep calm and carry on hacking.
+            pass
+
     console = Console(args.iface,
                       prompt=args.prompt,
                       monitor=monitor,
@@ -67,8 +85,12 @@ def create_depthcharge_ctx(args, **kwargs):
     if args.extra:
         kwargs = {**kwargs, **args.extra}
 
-    if hasattr(args, 'no_reboot'):
-        kwargs['allow_reboot'] = not args.no_reboot
+    # Arguments to pass to Depthcharge if non-None or True (for bools)
+    keys = ('arch', 'allow_deploy', 'skip_deploy', 'allow_reboot')
+
+    for key in keys:
+        if hasattr(args, key) and getattr(args, key):
+            kwargs[key] = getattr(args, key)
 
     if hasattr(args, 'config') and args.config:
         try:
@@ -76,7 +98,6 @@ def create_depthcharge_ctx(args, **kwargs):
             return Depthcharge.load(args.config,
                                     console,
                                     companion=companion,
-                                    skip_deploy=args.skip_deploy,
                                     **kwargs)
         except FileNotFoundError:
             # We'll create it when we call save()
@@ -84,7 +105,6 @@ def create_depthcharge_ctx(args, **kwargs):
 
     return Depthcharge(console,
                        companion=companion,
-                       skip_deploy=args.skip_deploy,
                        **kwargs)
 
 
@@ -191,14 +211,16 @@ class ArgumentParser(argparse.ArgumentParser):
     #: Default list used by :py:meth:`ArgumentParser.__init__()` unless
     #: otherwise overridden with a caller-provided list.
     DEFAULT_ARGS = [
+        'arch',
         'config',
         'interface',
         'companion',
         'monitor',
         'extra',
         'prompt',
+        'allow_deploy',
         'skip_deploy',
-        'no_reboot',
+        'allow_reboot',
     ]
 
     def _perform_arg_handler_init(self, init_args: list, kwargs_dict: dict):
@@ -301,94 +323,100 @@ class ArgumentParser(argparse.ArgumentParser):
         except AttributeError:
             pass
 
-    def add_address_argument(self, default=0, help=None, required=True):
+    def add_address_argument(self, **kwargs):
         """
         Add a memory address argument to the ArgumentParser.
         """
-        if help is None:
-            help = 'Base address of image.'
+        default_value = kwargs.pop('default', 0)
+        help_text = 'Base address of image.'
+        is_required = kwargs.pop('required', default_value is not None)
 
-        if default is not None:
-            help += ' Default: 0x{:08x}'.format(default)
+        if default_value is not None:
+            help_text += ' Default: 0x{:08x}'.format(default_value)
 
         self.add_argument('-a', '--address',
-                          metavar='<value>',
-                          default=default,
+                          metavar=kwargs.pop('metavar', '<value>'),
+                          default=kwargs.pop('default', default_value),
                           action=AddressAction,
-                          required=required,
-                          help=help)
+                          required=is_required,
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_arch_argument(self, default='ARM', required=False):
+    def add_arch_argument(self, **kwargs):
         """
         Add a CPU architecture argument to the ArgumentParser.
         """
-        self.add_argument('-A', '--arch',
-                          metavar='<architecture>',
-                          default=default,
-                          required=required,
-                          help='CPU architecture. Default: ' + default)
+        help_text = kwargs.pop('help', 'CPU architecture.')
+        self.add_argument('--arch',
+                          metavar=kwargs.pop('metavar', '<architecture>'),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_companion_argument(self, required=False):
+    def add_companion_argument(self, **kwargs):
         """
         Add an argument to the ArgumentParser that can be used to
         specify a Depthcharge Companion device and its corresponding
         settings.
         """
-        help = ('Depthcharge companion device to use and its associated settings.'
-                ' See the depthcharge.Companion documentation for supported '
-                ' settings.')
+        help_text = (
+            'Depthcharge companion device to use and its associated settings. '
+            'See the depthcharge.Companion documentation for supported settings.'
+        )
 
         self.add_argument('-C', '--companion',
-                          metavar='<device>[:setting=value,...]',
-                          default=None,
+                          metavar=kwargs.pop('metavar', '<device>[:setting=value,...]'),
+                          default=kwargs.pop('default', None),
                           action=CompanionAction,
-                          required=required,
-                          help=help)
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_config_argument(self, required=False, help=None):
+    def add_config_argument(self, **kwargs):
         """
         Add the device configuration argument to the ArgumentParser.
         """
-
-        if help is None:
-            help = ('Device configuration file to load and update. '
-                    'It will be created if it does not exist.')
+        help_text = (
+            'Device configuration file to load and update. '
+            'It will be created if it does not exist.'
+        )
 
         self.add_argument('-c', '--config',
-                          metavar='<cfg>',
-                          required=required,
-                          help=help)
+                          metavar=kwargs.pop('metavar', '<cfg>'),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_data_argument(self, help=None, required=False):
+    def add_data_argument(self, **kwargs):
         """
         Add an option to provide data as a hex string to the ArgumentParser.
 
         The caller is required to provide help text in order to describe the
         nature of the data users must provide.
         """
-        self.add_argument('-d', '--data', metavar='<hex str>', help=help, required=required)
+        self.add_argument('-d', '--data',
+                          metavar=kwargs.pop('metavar', '<hex str>'),
+                          **kwargs)
 
-    def add_extra_argument(self, help=None, required=False):
+    def add_extra_argument(self, **kwargs):
         """
         Add an option to allow scripts to provide "extra" arguments via the
         command line, which translate directly to ``**kwargs`` passed to
         Depthcharge API calls.
         """
-        if help is None:
-            help = ('Specify extra operation-specific parameters as a key-value '
-                    'pair. A value of True is implicit if a value is not '
-                    'explicitly provided. Multiple instances of this argument '
-                    'are permitted. See the documentation for subclasses of '
-                    'depthcharge.Operation for supported keyword arguments.')
+        help_text = (
+            'Specify extra operation-specific parameters as a key-value '
+            'pair. A value of True is implicit if a value is not '
+            'explicitly provided. Multiple instances of this argument '
+            'are permitted. See the documentation for subclasses of '
+            'depthcharge.Operation for supported keyword arguments.'
+        )
 
         self.add_argument('-X', '--extra',
-                          metavar='<key>[=<value>]',
+                          metavar=kwargs.pop('metavar', '<key>[=<value>]'),
                           action=KeyValListAction,
-                          required=required,
-                          default={},
-                          help=help)
+                          default=kwargs.pop('default', {}),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_file_argument(self, help=None, required=False):
+    def add_file_argument(self, **kwargs):
         """
         Add a file argument to the ArgumentParser.
 
@@ -398,70 +426,85 @@ class ArgumentParser(argparse.ArgumentParser):
         if help is None:
             raise ValueError('Help text must be provided for -f,--file ')
 
-        self.add_argument('-f', '--file', metavar='<path>', help=help, required=required)
+        self.add_argument('-f', '--file',
+                          metavar=kwargs.pop('metavar', '<path>'),
+                          **kwargs)
 
-    def add_interface_argument(self, required=False):
+    def add_interface_argument(self, **kwargs):
         """
         Add a serial console interface option the the ArgumentParser.
         """
+        help_text = 'Serial port interface connected to U-Boot console.'
         self.add_argument('-i', '--iface',
-                          metavar='<console dev>[:baudrate]',
-                          default='/dev/ttyUSB0:115200',
-                          required=required,
-                          help='Serial port interface connected to U-Boot console.')
+                          metavar=kwargs.pop('metavar', '<console dev>[:baudrate]'),
+                          default=kwargs.pop('default', _DEFAULT_IFACE),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_op_argument(self, required=False):
+    def add_op_argument(self, **kwargs):
         """
         Add an argument to allow desired :py:class:`depthcharge.Operation` implementations to be
         requested, by name.
         """
-        help = ('Request that one of the specified depthcharge.Operation '
-                'implementations be used. Depthcharge will attempt to choose '
-                'the best available option if this is not provided.')
+        help_text = (
+            'Request that one of the specified depthcharge.Operation '
+            'implementations be used. Depthcharge will attempt to choose '
+            'the best available option if this is not provided.'
+        )
 
         self.add_argument('--op',
-                          metavar='<name>[,name,...]',
-                          default=None,
+                          metavar=kwargs.pop('metavar', '<name>[,name,...]'),
+                          default=kwargs.pop('default', None),
                           action=ListAction,
-                          required=required,
-                          help=help)
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_length_argument(self, help=None, required=False):
+    def add_length_argument(self, **kwargs):
         """
         Add a length argument to the ArgumentParser.
 
         Default help text describes a read operation, in bytes.
         """
-        if help is None:
-            help = 'Number of bytes to read'
+        default_value = kwargs.pop('default', None)
+        is_required = kwargs.pop('required', default_value is None)
 
         self.add_argument('-l', '--length',
-                          default=None,
-                          metavar='<n>',
+                          default=default_value,
+                          required=is_required,
+                          metavar=kwargs.pop('metavar', '<n>'),
                           action=LengthAction,
-                          help=help,
-                          required=required)
+                          help=kwargs.pop('help', 'Number of bytes to read'),
+                          **kwargs)
 
-    def add_monitor_argument(self, required=False):
+    def add_monitor_argument(self, **kwargs):
         """
         Add a serial port monitor argument to the ArgumentParser.
         """
-        self.add_argument('-m',  '--monitor',
+        help_text = (
+            'Attach a console monitor. Valid types: file, pipe, colorpipe, term'
+        )
+
+        self.add_argument('-m', '--monitor',
                           metavar='<type>[:options,...]',
-                          default=None,
-                          required=required,
-                          help='Attach a console monitor. Valid types: file, pipe, colorpipe')
+                          default=kwargs.pop('default', None),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_no_reboot_argument(self, help=None):
-        if help is None:
-            help = 'Exclude operations that require crashing or rebooting the target.'
+    def add_allow_reboot_argument(self, **kwargs):
+        """
+        Add and argument that allows the user to opt-in to functionality that
+        necessitates crashing or rebooting the target platform.
+        """
+        default_help = ('Allow operations that require crashing or '
+                        'rebooting the target to be performed.')
 
-        self.add_argument('-R', '--no-reboot',
-                          default=False,
+        self.add_argument('-R', '--allow-reboot',
+                          default=kwargs.pop('default', False),
                           action='store_true',
-                          help=help)
+                          help=kwargs.pop('help', default_help),
+                          **kwargs)
 
-    def add_outfile_argument(self, help=None, required=False):
+    def add_outfile_argument(self, **kwargs):
         """
         Add an output file argument to the ArgumentParser.
 
@@ -473,36 +516,56 @@ class ArgumentParser(argparse.ArgumentParser):
 
         The caller is required to provide the *help* argument.
         """
-        if help is None:
+        if 'help' not in kwargs:
             raise ValueError('Help text must be provided for -o,--outfile ')
 
-        self.add_argument('-o', '--outfile', metavar='<path>', help=help, required=required)
+        self.add_argument('-o', '--outfile',
+                          metavar=kwargs.pop('meta', '<path>'),
+                          **kwargs)
 
-    def add_prompt_argument(self, required=False):
+    def add_prompt_argument(self, **kwargs):
         """
         Add an option to the ArgumentParser to allow for the expected U-Boot
         prompt to be supplied via the command line.
         """
-        help = 'Override expected U-Boot prompt string.'
+        help_text = 'Override expected U-Boot prompt string.'
         self.add_argument('-P', '--prompt',
                           metavar='<prompt str>',
-                          default=None,
-                          required=required,
-                          help=help)
+                          default=kwargs.pop('default', None),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
 
-    def add_skip_deploy_argument(self, required=False):
+    def add_allow_deploy_argument(self, **kwargs):
+        """
+        Add an opt-in option to the ArgumentParser that specifies that the user
+        wants to allow payload deployment and execution.
+        """
+        help_text = ('Allow payloads to be deployed and executed. '
+                     'Functionality may be limited if this is not specified.')
+
+        self.add_argument('-A', '--allow-deploy',
+                          action='store_true',
+                          default=kwargs.pop('default', False),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
+
+    def add_skip_deploy_argument(self, **kwargs):
         """
         Add an option to the ArgumentParser to allow payload deployment to be
         skipped in situations where a prior script or command has already
-        done this.
+        done this. (They still want to execute payloads.)
         """
-        self.add_argument('-D', '--skip-deploy',
-                          action='store_true',
-                          default=False,
-                          required=required,
-                          help="Skip any payload deployments; assume they're already loaded.")
+        help_text = ('Skip payload deployment but allow execution; '
+                     "assume payloads are already deployed and execute as-needed. "
+                     'This has no effect when -A, --allow-deploy is used.')
 
-    def add_stratagem_argument(self, help=None, metavar='<file>', required=False):
+        self.add_argument('-S', '--skip-deploy',
+                          action='store_true',
+                          default=kwargs.pop('default', False),
+                          help=kwargs.pop('help', help_text),
+                          **kwargs)
+
+    def add_stratagem_argument(self, **kwargs):
         """
         Add an option to the ArgumentParser to allow a user to supply a file
         containing a :py:class:`depthcharge.Stratagem` JSON file.
@@ -510,7 +573,9 @@ class ArgumentParser(argparse.ArgumentParser):
         The caller must provide help text specifying the purpose of the
         stratagem file (e.g. whether its being produced or is an input).
         """
-        if help is None:
+        if 'help' not in kwargs:
             raise ValueError('Help text must be provided for -s,--stratagem')
 
-        self.add_argument('-s', '--stratagem', metavar=metavar, help=help, required=required)
+        self.add_argument('-s', '--stratagem',
+                          metavar=kwargs.pop('metavar', '<file>'),
+                          **kwargs)
