@@ -116,24 +116,34 @@ class Depthcharge:
 
     **Keyword Arguments:**
 
-    :Detailed Help: When inspecting a device, the Depthcharge constructor will not
+    :Retrieve detailed help text: When inspecting a device, the Depthcharge constructor will not
         read detailed help text. This can be achieved by passing a *detailed_help=True*
         keyword argument to the constructor or by making a later call to
         :py:meth:`commands()` with *detailed=True*.
 
-    :Skip Payload Deployment: When performing the same operations on a device repeatedly,
-        the deployment of executable payloads may be redundant and waste time.
-        This can be disabled by providing a *skip_deploy=True* keyword
-        argument. Beware, however, that the target will crash if an attempt to
-        execute a payload that has not been previously deployed.
+    :Allow payload deployment & execution: If *allow_deploy=True*, Depthcharge will attempt
+        to deploy and execute payloads in memory, when neccessary. Specifying
+        *allow_deploy=True* will always force payloads to be deployed; this overrides
+        (and therefore ignores) whatever is set for the following *skip_deploy* parameter.
 
-    :Payload Location: By default, payloads are staged 32 MiB beyond the
+    :Skip payload deployment, but allow execution: When performing the same operations on a device
+        repeatedly, the deployment of executable payloads may be redundant and waste time.  This can
+        be disabled by providing a *skip_deploy=True* keyword argument.
+
+        **Important**, the target will crash if an attempt is made to execute a payload that has not
+        been previously deployed.
+
+        Specifying *skip_deploy=True` implies that one wishes to execute payloads.
+        If this is not true -- you want no payload deployment and no execution --
+        then *allow_deploy=False* is what you're looking for.
+
+    :Payload location: By default, payloads are staged 32 MiB beyond the
         target's ``$loadaddr`` location. To override this location, either
         alternative absolute address can be provided in a *payload_base*
-        keyword argument.  Alternatively, the offset from ``$loadaddr`` can be
-        provided via *payload_offset*.
+        keyword argument.  Alternatively, the offset from the payload base address can
+        be provided via *payload_offset*.
 
-    :Crash/Reboot Behavior: Some operations, such as
+    :Crash/Reboot behavior: Some operations, such as
         :py:class:`~depthcharge.register.DataAbortRegisterReader` subclasses,
         need to crash platform (assuming it will automatically reboot) in order
         to perform their duty. If crashing or rebooting the platform is
@@ -150,6 +160,7 @@ class Depthcharge:
         constructor, specify the string ``'self'``.
     """
 
+    _default_arch = 'generic'  # 32-bit, little endian
     _ver_re   = re.compile(r'^U-Boot\s+[0-9]{4}\.[0-9]{2}')
 
     def __init__(self, console, companion=None, **kwargs):
@@ -163,10 +174,11 @@ class Depthcharge:
         else:
             self.console = console
 
-        self.arch = Architecture.get(kwargs.get('arch', 'arm'))
+        self.arch = Architecture.get(kwargs.get('arch', self._default_arch) or self._default_arch)
+        log.debug('Architecture: ' + self.arch.description)
 
         # Is the user allowing us to reboot or reset the platform?
-        self._allow_reboot = kwargs.get('allow_reboot', True)
+        self._allow_reboot = kwargs.get('allow_reboot', False)
 
         self._post_reboot_cb = kwargs.get('post_reboot_cb', None)
         self._post_reboot_cb_data = kwargs.get('post_reboot_cb_data', None)
@@ -218,16 +230,54 @@ class Depthcharge:
         # to collect information.
         self._gd = kwargs.get('_gd', {})
 
-        # PayloadMap and its base address initialization may depend upon
-        # information gathered on the target.
+        # Are we permitted to deploy executable payloads as-needed?
+        self._allow_deploy_exec = kwargs.get('allow_deploy', False)
+
+        # Should we assume that payloads are already deployed?
+        self._skip_deployment = kwargs.get('skip_deploy', False)
+
+        # Reconcile desired behavior, based upon what was explicitly
+        # specified, implied, or being defaulted. See the function doc comments.
+
+        if 'allow_deploy' in kwargs:  # User specified, not default value
+            self._skip_deployment = False
+            # For any explicit usage, whether True or False, allow_deploy takes precendence and
+            # resuls in deployment + exec skip_deploy only has an effect when the user does not
+            # specify this param.
+
+        elif self._skip_deployment:
+            # Skipping deployment still implies exec
+            self._allow_deploy_exec = True
+
+        # Our payload map will be initializde during the following active
+        # initialization portion of this constructor.
         self._payloads = None
-        self._payload_base = None
+
+        # Where should we place our payloads?
+        # If not specified, we'll used an offset from ${loadaddr} as to avoid
+        # stomping on stuff that we actively may wish to tamper wtih.
+        self._payload_base = kwargs.get('payload_base', 'loadaddr')
+        if 'payload_base' not in kwargs:  # We used the loadaddr default...
+            log.note('Using default payload base address: ${loadaddr} + 32MiB')
+            self._payload_off = 32 * 1024 * 1024
+        else:
+            self._payload_off = kwargs.get('payload_offset', 0)
 
         # Perform initializations involve interaction with the underlying
         # device. This has been split just to afford us an opportunity to
         # suppress or defer this, should such an API change ever be necessary.
         self._perform_active_init(**kwargs)
 
+        # With some future additions, we could either directly determine
+        # or deduce the architecture here, if we're still using a Generic* arch.
+        #
+        # From there, we can re-init accordingly.
+        #
+        # For now, just warn that functionality may be limited with Generic*
+        if 'Generic' in self.arch.name:
+            m = 'Using {:s} architecture. Functionality may be limited without more specific architecture.'
+            log.warning(m.format(self.arch.name))
+
     def _perform_active_init(self, **kwargs):
         """
         Helper for __init__() that performs active initialization operations.
@@ -243,7 +293,16 @@ class Depthcharge:
 
         # Establish our payload base address by resolving either an environment
         # variable read from the target device or using a user-provided address
-        self._setup_payload_map(**kwargs)
+        try:
+            payload_map_base = self._resolve_payload_base()
+        except OperationFailed:
+            self._allow_deploy_exec = False
+            self._skip_deploy = False
+            payload_map_base = 0
+            log.warning('Disabling payload deployemnt and execution due to error(s).')
+
+        self._payloads = PayloadMap(self.arch, payload_map_base,
+                                    skip_deploy=self._skip_deployment)
 
         # Retrieve and cache version information, if not provided earlier
         self.version()
@@ -265,40 +324,32 @@ class Depthcharge:
         except OperationNotSupported as error:
             log.warning(str(error))
 
-    def _setup_payload_map(self, **kwargs):
+    def _resolve_payload_base(self):
         """
-        Set up payload map using either:
-
-            * keyword argument: payload_base
-            * U-Boot target environment variable: loadaddr
+        Resolve self._payload_base string based upon environment vars, if needed.
         """
-        payload_base = kwargs.get('payload_base', 'loadaddr')
-        if isinstance(payload_base,  str):
-            if payload_base not in self._env:
-                msg = 'Did not find environment variable specified for payload_base ("{:s}")'
-                raise ValueError(msg.format(payload_base))
+        if isinstance(self._payload_base,  str):
+            try:
+                expanded = uboot.env.expand_variable(self._env, self._payload_base)
+                self._payload_base = int(expanded, 0)
+            except KeyError:
+                msg = 'Environment variable used for payload_base does not exist: {:s}'
+                raise OperationFailed(msg.format(self._payload_base))
+            except ValueError:
+                msg = 'Encountered invalid expansion of payload_base: ' + expanded
+                raise OperationFailed(msg)
 
-            # We are usually interested in (tampering with) what gets loaded @
-            # ${loadaddr}, so don't stage payloads there.
-            apply_offset = (payload_base == 'loadaddr')
-            payload_base = int(self._env[payload_base], 0)
-            if apply_offset:
-                payload_base += kwargs.get('payload_offset', 32 * 1024 * 1024)
-        else:
-            payload_base = int(payload_base)
-
-        # Set up payload memory map
-        skip_deploy = kwargs.get('skip_deploy', False)
-        self._payload_base = payload_base
-        log.note('Depthcharge PayloadMap base address: 0x{:08x}'.format(payload_base))
-        self._payloads = PayloadMap(self.arch, self._payload_base, skip_deploy=skip_deploy)
+        msg = 'Depthcharge payload base (0x{:08x}) + payload offset (0x{:08x}) => 0x{:08x}'
+        actual_base = self._payload_base + self._payload_off
+        log.note(msg.format(self._payload_base, self._payload_off, actual_base))
+        return actual_base
 
     @staticmethod
     def _log_not_supported(err):
         # Warn for situations where more functionality would be available if
         # the user had provided
         msg = '  Excluded:  ' + str(err)
-        if 'companion' in msg:
+        if 'companion' in msg or 'opt-in not specified' in msg:
             log.warning(msg)
         else:
             log.note(msg)
@@ -348,13 +399,22 @@ class Depthcharge:
             try:
                 impl = subclass(self, **kwargs)
                 required_payloads = impl.required['payloads']
-                if len(required_payloads) > 0 and len(self._memwr) == 0:
-                    msg = 'No MemoryWriter available to deploy required payload(s)'
-                    raise OperationNotSupported(impl, msg)
+                if len(required_payloads) > 0:
+                    if len(self._memwr) == 0:
+                        msg = 'No MemoryWriter available to deploy required payload(s)'
+                        raise OperationNotSupported(impl, msg)
+
+                    if not self._allow_deploy_exec:
+                        msg = 'Payload deployment+execution opt-in not specified'
+                        raise OperationNotSupported(impl, msg)
 
                 for payload in required_payloads:
                     self._payloads.mark_required_by(payload, impl)
 
+                if issubclass(subclass, Executor) and not self._allow_deploy_exec:
+                    msg = 'Payload deployment+execution opt-in not specified'
+                    raise OperationNotSupported(impl, msg)
+
                 op_set.add(impl)
                 log.note('  Available: ' + impl.name)
 
@@ -450,13 +510,20 @@ class Depthcharge:
         previously created by :py:meth:`to_json()`.
         """
         ctx = json.loads(json_str)
+
+        # Allow these items to be overridden on load
+        if 'payload_base' not in kwargs and 'payload_base' in ctx:
+            kwargs['payload_base'] = ctx['payload_base']
+
+        if 'payload_offset' not in kwargs and 'payload_offset' in ctx:
+            kwargs['payload_offset'] = ctx['payload_offset']
+
         return cls(console,
-                   arch=ctx['arch'],
+                   arch=kwargs.pop('arch', ctx['arch']),
                    _version=ctx['version'],
                    _cmds=ctx['commands'],
                    _env=ctx['env_vars'],
                    _prompt=ctx['prompt'],
-                   payload_base=ctx['payload_base'],
                    _gd=ctx['gd'],
                    **kwargs)
 
@@ -467,11 +534,13 @@ class Depthcharge:
         """
         output = {
             'arch': self.arch.name,
+            'baudrate': self.console.baudrate,
             'version': self._version,
             'prompt': self.console.prompt,
             'commands': self._cmds,
             'env_vars': self._env,
             'payload_base': self._payload_base,
+            'payload_offset': self._payload_off,
             'gd': self._gd,
         }
 
@@ -494,7 +563,7 @@ class Depthcharge:
         Serialize the current configuration of the current Depthcharge context
         to a JSON object and write it to the provided filename.
         """
-        log.info('Saving depthcharge configuration state to ' + filename)
+        log.note('Saving depthcharge configuration state to ' + filename)
         s = self.to_json(timestamp, comment)
         with open(filename, 'w') as outfile:
             outfile.write(s)
@@ -654,6 +723,8 @@ class Depthcharge:
 
         Note that all values in the returned dictionary are strings. The caller is responsible for
         performing any necessary type conversions.
+
+        Upon failure, an error will be logged and an empty dictionary will be returned.
         """
 
         if self._env is not None and cached:
@@ -664,7 +735,12 @@ class Depthcharge:
         self.console.interrupt()
         env_text = self.send_command('printenv')
 
-        self._env = uboot.parse_environment(env_text)
+        try:
+            self._env = uboot.env.parse(env_text)
+        except ValueError as e:
+            log.error('Failed to parse environment: ' + str(e))
+            self._env = {}
+
         return self._env
 
     def env_var(self, name: str, expand=True, cached=True, convert_int=True, **kwargs):
@@ -688,7 +764,7 @@ class Depthcharge:
             self.environment(cached=False)
 
         if expand:
-            ret = uboot.expand_variable(self._env, name, **kwargs)
+            ret = uboot.env.expand_variable(self._env, name, **kwargs)
         else:
             ret = self._env[name]
 
@@ -751,7 +827,7 @@ class Depthcharge:
             resp = self.console.send_command('version')
             self._version = resp.splitlines()
             if len(self._version) >= 1:
-                log.info('Version: ' + self._version[0])
+                log.note('Version: ' + self._version[0])
             return self._version
 
         if 'reset' in self._cmds and allow_reset:
@@ -761,7 +837,7 @@ class Depthcharge:
             for line in lines:
                 if self._ver_re.match(line):
                     self._version = [line.strip()]
-                    log.info('Version: ' + self._version[0])
+                    log.note('Version: ' + self._version[0])
                     return self._version
 
             log.warning('Did not see U-Boot version string. Old or non-standard version format?')
@@ -1050,6 +1126,9 @@ class Depthcharge:
 
         The keyword argument *force=True* can be used to force (re)deployment.
         """
+        if self._allow_deploy_exec is False:
+            # Shouldn't happen
+            raise OperationFailed('Not performing payload deployment. Requires opt-in.')
 
         payload     = self._payloads[name]
         addr        = payload['address']
@@ -1063,7 +1142,7 @@ class Depthcharge:
             return
 
         if not deployed or force:
-            log.info('Deploying payload \"{:s}\" @ 0x{:08x}'.format(name, addr))
+            log.note('Deploying payload \"{:s}\" @ 0x{:08x}'.format(name, addr))
             self.write_memory(addr, payload['data'])
             self._payloads.mark_deployed(name)
 
@@ -1085,6 +1164,11 @@ class Depthcharge:
         reading of response data, should the caller want to do so manually
         through raw console accesses. In this case, this method returns ``None``.
         """
+        if self._allow_deploy_exec is False:
+            msg = ('Not attempting payload execution. '
+                   'Requires opt-in of payload deployment and execution.')
+            raise OperationFailed(msg)
+
         payload = self._payloads[name]
 
         # Deploy payload as-needed. (No-op if already deployed.)
@@ -1164,11 +1248,11 @@ class Depthcharge:
 
         if already_applied == len(patch_list):
             msg = 'Target memory appears to be already patched. No writes needed.'
-            log.info(msg)
+            log.note(msg)
             return False
 
         msg = 'Target memory appears to have been only partially patched.'
-        log.info(msg)
+        log.note(msg)
         return True
 
     def uboot_global_data(self, cached=True, **kwargs) -> dict:
@@ -1214,7 +1298,7 @@ class Depthcharge:
 
         if 'bdinfo' in self._cmds and not kwargs.get('skip_bdinfo', False):
             resp = self.console.send_command('bdinfo')
-            bdinfo = uboot.bdinfo_dict(resp)
+            bdinfo = uboot.board.bdinfo_dict(resp)
             if bdinfo:  # Don't insert any empty dict if things go awry
                 self._gd['bd'] = bdinfo
         else:
@@ -1263,7 +1347,7 @@ class Depthcharge:
 
             self._gd['address'] = register_reader.read(gd_reg)
             msg = 'Located U-boot global data structure (*gd) @ 0x{:08x}'
-            log.info(msg.format(self._gd['address']))
+            log.note(msg.format(self._gd['address']))
             return self._gd['address']
 
     def _uboot_jump_table(self, memory_reader=None):
@@ -1279,5 +1363,5 @@ class Depthcharge:
         if memory_reader is None:
             memory_reader = self._memrd.default()
 
-        self._gd['jt'] = uboot.find_jump_table(self._gd['address'], memory_reader, self.arch)
+        self._gd['jt'] = uboot.jump_table.find(self._gd['address'], memory_reader, self.arch)
         return self._gd['jt']
